/******************************************************************
*************************Classe BSTNode****************************
******************************************************************/
class BSTNode
{
	private Object key;
	private BSTNode left, right;

	//costruttore senza argomenti
	public BSTNode ()
	{
		left = right = null;
	}

	//costruttore con un argomento
	public BSTNode (Object val)
	{
		this (val, null, null); //chiama il terzo costruttore
	}

	//Costruttore:
	public BSTNode (Object val, BSTNode lt, BSTNode rt)
	{
		key = val;
		left = lt;
		right = rt;
	}

	//metodo per la visita del nodo
	public Object visit ()
	{
		return key;
	}

	//metodi di accesso/modifica
	public Object getKey ()
	{
		return key;
	}

	public BSTNode getLeft()
	{
		return left;
	}

	public BSTNode getRight()
	{
		return right;
	}

	public void setKey (Object val)
	{
		key = val;
	}

	public void setLeft (BSTNode lt)
	{
		left = lt;
	}

	public void setRight (BSTNode rt)
	{
		right = rt;
	}
}

/******************************************************************
***************************Classe BST******************************
******************************************************************/
public class BST
{
	protected BSTNode root;

	//Costruttore (crea un albero vuoto)
	public BST ()
	{
		root = null;
	}

	//metodo che verifica se il BST  vuoto
	public boolean isEmpty ()
	{
		return (root == null);
	}

	public boolean isEmpty (BSTNode p)
	{
		return (p == null);
	}

	//metodo che restituisce il riferimento alla radice dell'albero
	public BSTNode getRoot()
	{
		return root;
	}

	/******************* Metodi per l'ATTRAVERSAMENTO *******************/

	//attraversamento PREORDER ricorsivo (radice, sottoalbero sx, sottoalbero dx)
	public void preorder (BSTNode p)
   {
		if (p != null)
		{
			p.visit ();
			preorder (p.getLeft());
			preorder (p.getRight());
		}
	}

	//attraversamento PREORDER iterativo
	public void iterativePreorder (BSTNode p)
	{
		StackList supp = new StackList();
		if (p != null)
		{
			supp.push (p);
			while (!supp.isEmpty())
			{
				p.visit();
				if (p.getRight() != null)
					supp.push (p.getRight());
				if (p.getLeft() != null)
					supp.push (p.getLeft());
			}
		}
	}

	//attraversamento INORDER ricorsivo (sottoalbero sx, radice, sottoalbero dx)
	public void inorder (BSTNode p)
   {
		if (p != null)
		{
			inorder (p.getLeft());
			p.visit();
			inorder (p.getRight());
		}
	}

	//attraversamento INORDER iterativo 
	public void iterativeInorder (BSTNode p)
	{
		StackList supp = new StackList ();
		while (p != null)
		{
			if (p.getRight() != null)
				supp.push (p);
			supp.push (p);
			p = p.getLeft();
		}
		p = (BSTNode) supp.pop();
		while (!supp.isEmpty() && p.getRight() == null)
		{
			p.visit();
			p = (BSTNode) supp.pop();
		}
		p.visit();
		if (!supp.isEmpty())
			p = (BSTNode) supp.pop();
		else
			p = null;
	}

	//attraversamento POSTORDER ricorsivo (sottoalbero sx, sottoalbero dx, radice)
	public void postorder (BSTNode p)
   {
		if (p != null)
		{
			postorder (p.getLeft());
			postorder (p.getRight());
			p.visit ();
		}
	}

	//attarversamento POSTORDER iterativo
	public void iterativePostorder (BSTNode p)
	{
		BSTNode q = p;
		StackList supp = new StackList();
		while (p != null)
		{
			for ( ; p.getLeft() != null; p = p.getLeft())
				supp.push (p);
			while (p != null && (p.getRight() == null || p.getRight () == q))
			{
				p.visit();
				q = p;
				if (supp.isEmpty())
					return;
				p = (BSTNode) supp.pop();
			}
			supp.push (p);
			p = p.getRight();
		}
	}

	/******************* Metodi per la RICERCA *******************/
	
	//ricerca (iterativa):
	public BSTNode iterativeSearch (Object val)
	{
		BSTNode p = root;
		while (p != null)
		{
			if (((Comparable)p.getKey()).equals(val))
				return p;
			else
			{
				if (((Comparable)p.getKey()).compareTo(val) > 0)
					p = p.getLeft();
				else
					p = p.getRight();
			}
		}
		return null;
	}
	
	public BSTNode iterativeSearch (BSTNode p, Object val)
	{
		while (p != null)
		{
			if (((Comparable)p.getKey()).equals(val))
				return p;
			else
			{
				if (((Comparable)p.getKey()).compareTo(val) > 0)
					p = p.getLeft();
				else
					p = p.getRight();
			}
		}
		return null;
	}

	//ricerca (ricorsiva):
	public BSTNode search (BSTNode p, Object val)
	{
		if (p == null)
			return null;
		if (((Comparable)p.getKey()).equals(val))
			return p;
		else if (((Comparable)p.getKey()).compareTo(val) > 0)
			return this.search (p.getLeft(), val);
		else
			return this.search (p.getRight(), val);
	}

	/******************* Metodi per l'INSERIMENTO *******************/
	
	//inserimento (iterativo):
	public void insert (Object val)
	{
		if (root == null)		//BST vuoto
			root = new BSTNode (val);
		else		//BST non vuoto
		{
			BSTNode p = root , prev = null;
			while (p != null)
			{
				if (((Comparable)p.getKey()).equals(val))	//l'elemento gi esiste
					throw new NodeFoundException();
				prev = p;
				if (((Comparable)p.getKey()).compareTo(val) > 0)
					p = p.getLeft();
				else if (((Comparable)p.getKey()).compareTo(val) < 0)
					p = p.getRight();
			}
			// possibile inserire l'elemento:
			if (((Comparable)prev.getKey()).compareTo(val) > 0)
				prev.setLeft (new BSTNode (val));
			else
				prev.setRight (new BSTNode (val));
		}
	}

	//inserimento (ricorsivo):
	public void insert (BSTNode p, Object val) //overloading!
	{
		if (root == null)
			root = new BSTNode (val);
		else if (p.getLeft() == null || p.getRight() == null)
		{
			if (((Comparable)p.getKey()).compareTo(val) > 0)
				p.setLeft (new BSTNode (val));
			else
				p.setRight (new BSTNode (val));
		}
		else
		{
			if (((Comparable)p.getKey()).compareTo(val) > 0)
				this.insert (p.getLeft(), val);
			else
				this.insert (p.getRight(), val);
		}
	}

	/******************* Metodo per la CANCELLAZIONE *******************/
	public BSTNode delete (Object val)
	{
		if (root == null) 	//l'albero  vuoto
			throw new EmptyTreeException();
		BSTNode nodo, p = root, prev = null;
		while (p != null && !((Comparable)p.getKey()).equals(val)) 	//ricerca dell'elemento
		{
			prev = p; 	//tengo il riferimento del precedente
			if (((Comparable)p.getKey()).compareTo(val) > 0)
				p = p.getLeft();
			else
				p = p.getRight();
		}
		nodo = p;
		if (p != null && ((Comparable)p.getKey()).equals(val)) // stato trovato l'elemento da cancellare
		{
			if (nodo.getRight() == null) 	//il nodo non ha figlio dx
				nodo = nodo.getLeft();
			else if (nodo.getLeft() == null) 	//il nodo non ha figlio sx
				nodo = nodo.getRight();
			else 	//il nodo ha ambedue i figli
			{
				/*********FONDISOTTOALBERI***********/
				//ricerca del predecessore:
				BSTNode tmp = nodo.getLeft();
				while (tmp.getRight() != null)
					tmp = tmp.getRight();
				//fusione:
				tmp.setRight (nodo.getRight());
				nodo = nodo.getLeft();
				/************************************/
				/*********SOSTITUISCICHIAVE**********/
					/*
					//ricerca del predecessore:
					BSTNode tmp = nodo.getLeft();
					BSTNode previous = nodo;
					while (tmp.getRight() != null)
					{
						previous = tmp;
						tmp = tmp.getRight();
					}
					nodo.setKey (tmp.getKey()); //sostituzione della chiave
					if (previous == nodo)
						previous.setLeft (tmp.getLeft());
					else
						previous.setRight (tmp.getRight());
					*/
					/********************************/
			}
			if (p == root)
				root = nodo;
			else if (p.getLeft() == p)
				prev.setLeft(nodo);
			else
				prev.setRight(nodo);
			return p;
		}
		return null;
	}

	//ESERCIZIO B1: metodo ricorsivo che restituisce l'altezza del BST
	public int getHeight (BSTNode p)
	{
		int h = 0;
		if (p == null)
			return 0;
		else
		{
			int sx = 1 + getHeight (p.getLeft());
			int dx = 1 + getHeight (p.getRight());
			h = Math.max (sx, dx);
		}
		if (p == this.getRoot())
			return h-1;
		return h;
	}

	//ESERCIZIO B2: metodo ricorsivo che restituisce il numero di foglie del BST
	public int getNumFoglie (BSTNode p)
	{
		int c = 0;
		if (p != null)
		{
			if (p.getLeft() == null && p.getRight() == null)
				return 1;
			else
			{
				c += getNumFoglie (p.getLeft());
				c += getNumFoglie (p.getRight());
			}
		}
		return c;
	}

	//Metodo ricorsivo che restituisce il numero di nodi del BST:
	public int getNumNodi (BSTNode p)
	{
		if (p == null)
			return 0;
		else return 1 + getNumNodi (p.getLeft()) + getNumNodi (p.getRight());
	}

	//ESERCIZIO B8: metodo che dato in input un nodo restituisce il nodo fratello
	public BSTNode getBrother (BSTNode p)
	{
		BSTNode aux = root, prev = null;
		while (aux != null)
		{
			if (p == aux)
			{
				if (prev.getRight() == p)
					return prev.getLeft();
				else return prev.getRight();
			}
			prev = aux;
			if (((Comparable)p.getKey()).compareTo(aux.getKey()) < 0)
				aux = aux.getLeft();
			else aux = aux.getRight();
		}
		return null;
	}
}

