//Lista circolare semplicemente concatenata di INTERI

public class CircularList
{
	protected Node head;

	//Costruttore:
	public CircularList ()
	{
		head = null; 
	}

	//metodo che verifica se la lista  vuota
	public boolean isEmpty ()
	{
		return (head == null); 
	}

	public Node getHead ()
	{
		return head;
	}

	public void setHead (Node n)
	{
		head = n;
	}

	public int getLength ()
	{
		int cont = 1;
		Node aux = head;
		for ( ; aux.getNext() != head; cont++, aux = aux.getNext()) ;
		return cont;
	}

	public void clear ()
	{
		head = null;
	}

	//inserimento in testa
	public void insertHead (Object obj)
	{
   	if (isEmpty())
		{
			head = new Node (obj);
			head.setNext(head);
		}
		else
		{
			//prima scorro la lista fino a raggiungere la coda
			Node aux = head;
			for ( ; aux.getNext() != head; aux = aux.getNext()) ;
			//aggancio l'elemento
			head = new Node (obj, head);
			//cambio il riferimento alla coda
			aux.setNext (head);
		}
	}

	//inserimento in coda
	public void insertTail (Object obj)
	{
		if (isEmpty())
			insertHead (obj);
		else
		{
		//scansione della lista fino alla coda
		Node aux = head;
		for ( ; aux.getNext() != head; aux = aux.getNext()) ;
		aux.setNext (new Node (obj, head));
		}
	}
	
	//inserimento ordinato 
	public void insertOrdered (Object obj)
	{
		if (isEmpty() || ((Comparable)obj).compareTo(head.getInfo()) < 0)
			insertHead (obj);
		else
		{
			//scansione della lista
			Node aux = head;
			for ( ; aux.getNext() != head && ((Comparable)obj).compareTo(aux.getNext().getInfo()) > 0; aux = aux.getNext()) ;
			aux.setNext (new Node (obj, aux.getNext()));
		}
	}

	//ricerca ordinata
	public Node searchOrd (Object key)				 //key  la chiave di ricerca
	{
		if (key.equals( head.getInfo()))
			return head;
		else
		{
			Node aux = head.getNext();
			for (; aux != head && ((Comparable)aux.getInfo()).compareTo(key) < 0; aux = aux.getNext()) ;
			if (aux != null && aux.getInfo().equals(key))
				return aux;
			return null;	//nel caso in cui l'elemento da ricercare non fosse stato trovato
		}
	}

	//ricerca non ordinata
	public Node search (Object key)
	{
		if (key.equals(head.getInfo()))
			return head;
		else
		{
			Node aux = head.getNext();
			for (; aux != head && !(aux.getInfo().equals(key)); aux = aux.getNext()) ;
			return aux;
		}
	}
 
	//cancellazione della testa
	public Object deleteHead ()
	{
		Object ret = null;
		if (isEmpty())
			throw new EmptyListException();
		else if (head == head.getNext())
			head = null;
		else
		{
			//scansione della lista
			Node aux = head;
			for ( ; aux.getNext() != head; aux = aux.getNext()) ; //scansione
			ret = head.getInfo();
			head = head.getNext(); //cancello il riferimento della testa
			aux.setNext(head); //modifico il riferimento della coda
		}
		return ret;
	}

	//cancellazione della coda
	public Object deleteTail ()
	{
		if (isEmpty())
			throw new EmptyListException();
		else
		{
			Node aux = head;
			Node prev = null; //precedente di aux
			//scansione della lista
			for ( ; aux.getNext() != head; prev = aux, aux = aux.getNext()) ;
			if (prev == head) //la lista ha un solo elemento
				head = null;
			else //la lista ha pi di un elemento
				prev.setNext(head);
			return aux.getInfo();
		}
	}

	//cancellazione di una chiave specifica della lista
	public Node deleteKey (Object key)
	{
		Node aux = head.getNext();
		if (isEmpty())
			throw new EmptyListException();
		else if (key.equals(head.getInfo())) //l'elemento da cancellare sar quello di testa
			deleteHead();
		else
		{
			Node prev = head;
			//scansione
			for ( ; aux != head && !(aux.getInfo().equals(key)); prev = aux, aux = aux.getNext()) ;
			prev.setNext (aux.getNext());
		}
		return aux;
	}
}
