//Implementazione di una lista doppiamente linkata circolare generica

public class CircularListDll
{
	protected NodeDll head, tail;

	//Costruttore:
	public CircularListDll ()
	{
		head = tail = null; 
	}

	//metodo che verifica se la lista  vuota
	public boolean isEmpty()
	{
		return (head == null); 
	}

	//metodo che restituisce il riferimento alla testa della lista
	public NodeDll getHead ()
	{
		return head; 
	}

	public void setHead (NodeDll n)
	{
		head = n;
	}

	//metodo che restituisce il riferimento alla coda della lista
	public NodeDll getTail ()
	{
		return tail; 
	}

	public void setTail (NodeDll n)
	{
		tail = n;
	}

	//metodo che cancella la lista
	public void clear ()
	{
		head = tail = null; 
	}

	public int getLength ()
	{
		int cont = 1;
		NodeDll aux = head;
		for ( ; aux.getNext() != head; cont++, aux = aux.getNext()) ;
		return cont;
	}

	/*		METODI PER L'INSERIMENTO	*/
	//inserimento in testa
	public void insertHead (Object obj)
	{
		if (isEmpty())
		{
			head = tail = new NodeDll (obj);
			tail.setNext (head);
			head.setPrev (tail);
		}
		else
		{
			head = new NodeDll (obj, tail, head);
			head.getNext().setPrev (head);
			tail.setNext (head);
		}
	}
	
	//inserimento in coda
	public void insertTail (Object obj)
	{
		if (isEmpty())
			insertHead(obj);
		else
		{
			tail = new NodeDll (obj, tail, head);
			tail.getPrev().setNext (tail);
			head.setPrev (tail);
		}
	}

	//inserimento ordinato
	public void insertOrdered (Object obj)
	{
		if (isEmpty() || !(((Comparable)(head.getInfo())).compareTo(obj) < 0))
			insertHead (obj);
		else
		{
			NodeDll aux = head;
			for( ; aux != tail && ((Comparable)(aux.getInfo())).compareTo(obj) < 0; aux = aux.getNext()) ;
			if (aux == tail)
				insertTail (obj);
			else
			{
				aux.setPrev (new NodeDll (obj, aux.getPrev(), aux));
				aux.getPrev().getPrev().setNext (aux.getPrev());
			}
		}
	}
	/*		METODI PER LA RICERCA	*/
	//ricerca su lista ordinata
	public NodeDll searchOrd (Object key)
	{
		if (((Comparable)(head.getInfo())).equals(key))
			return head;
		else
		{
			NodeDll aux = head.getNext();
			for ( ; aux != head && ((Comparable)(aux.getInfo())).compareTo(key) < 0; aux = aux.getNext()) ;
			if (aux != head && ((Comparable)(aux.getInfo())).equals(key))
				return aux;
		}
		return null;
	}

	//ricerca su lista non ordinata
	public NodeDll search (Object key)
	{
		if (((Comparable)(head.getInfo())).equals(key))
			return head;
		else
		{
			NodeDll aux = head.getNext();
			for ( ; aux != head && !((Comparable)(aux.getInfo())).equals(key); aux = aux.getNext()) ;
			if (aux != head && ((Comparable)(aux.getInfo())).equals(key))
				return aux;
		}
		return null;
	}
	
	/*		METODI PER LA CANCELLAZIONE	*/
	//cancellazione della testa
	public Object deleteHead ()
	{
		if (isEmpty())
			throw new EmptyListException();
		Object obj = head.getInfo();
		if (head == head.getNext())
			head = tail = null;
		else
		{
			head = head.getNext();
			head.setPrev (tail);
			tail.setNext (head);
		}
		return obj;
	}

	//cancellazione della coda
	public Object deleteTail ()
	{
		if (isEmpty())
			throw new EmptyListException();
		Object obj = tail.getInfo();
		if (head == head.getNext())
			head = tail = null;
		else
		{
			tail = tail.getPrev();
			tail.setNext (head);
			head.setPrev (tail);
		}
		return obj;
	}

	//cancellazione di un chiave specifica
	public NodeDll deleteKey (Object key)
	{
		if (isEmpty())
			throw new EmptyListException();
		NodeDll aux = head;
		//l'elemento da cancellare  quello di testa
		if (((Comparable)(head.getInfo())).equals(key))
		{
			if (head.getNext() == head)
			{
				head = tail = null;
				return aux;
			}
			else
			{
				deleteHead();
				return aux;
			}
		}
		//l'elemento da cancellare non  di testa
		if (head == head.getNext())
			return null;
		aux = aux.getNext();
		for ( ; aux != head && !((Comparable)(aux.getInfo())).equals(key); aux = aux.getNext()) ;
		if (aux != head) // stato trovato l'elemento ricarcato
		{
			if (aux == tail) // l'elemento di cada
			{
				deleteTail ();
				return aux;
			}
			else // un altro elemento
			{
				aux.getPrev().setNext (aux.getNext());
				aux.getNext().setPrev (aux.getPrev());
				return aux;
			}
		}
		return null;
	}
}
