/******************************************************************
************************Classe IntBSTNode**************************
******************************************************************/
class IntBSTNode
{
	private int key;
	private IntBSTNode left, right;

	//costruttore senza argomenti
	public IntBSTNode ()
	{
		left = right = null;
	}

	//costruttore con un argomento
	public IntBSTNode (int val)
	{
		this (val, null, null); //chiama il terzo costruttore
	}

	//Costruttore:
	public IntBSTNode (int val, IntBSTNode lt, IntBSTNode rt)
	{
		key = val;
		left = lt;
		right = rt;
	}

	//metodo per la visita del nodo
	public int visit ()
	{
		return key;
	}

	//metodi di accesso/modifica
	public int getKey ()
	{
		return key;
	}

	public IntBSTNode getLeft()
	{
		return left;
	}

	public IntBSTNode getRight()
	{
		return right;
	}

	public void setKey (int val)
	{
		key = val;
	}

	public void setLeft (IntBSTNode lt)
	{
		left = lt;
	}

	public void setRight (IntBSTNode rt)
	{
		right = rt;
	}
}

/******************************************************************
**************************Classe IntBST****************************
******************************************************************/
public class IntBST
{
	protected IntBSTNode root;

	//Costruttore (crea un albero vuoto)
	public IntBST ()
	{
		root = null;
	}

	//metodo che verifica se il BST  vuoto
	public boolean isEmpty ()
	{
		return (root == null);
	}

	public boolean isEmpty (IntBSTNode p)
	{
		return (p == null);
	}

	//metodo che restituisce il riferimento alla radice dell'albero
	public IntBSTNode getRoot()
	{
		return root;
	}

	/******************* Metodi per l'ATTRAVERSAMENTO *******************/

	//attraversamento PREORDER ricorsivo (radice, sottoalbero sx, sottoalbero dx)
	public void preorder (IntBSTNode p)
   {
		if (p != null)
		{
			p.visit ();
			preorder (p.getLeft());
			preorder (p.getRight());
		}
	}

	//attraversamento PREORDER iterativo
	public void iterativePreorder (IntBSTNode p)
	{
		StackList supp = new StackList();
		if (p != null)
		{
			supp.push (p);
			while (!supp.isEmpty())
			{
				p.visit();
				if (p.getRight() != null)
					supp.push (p.getRight());
				if (p.getLeft() != null)
					supp.push (p.getLeft());
			}
		}
	}

	//attraversamento INORDER ricorsivo (sottoalbero sx, radice, sottoalbero dx)
	public void inorder (IntBSTNode p)
   {
		if (p != null)
		{
			inorder (p.getLeft());
			p.visit();
			inorder (p.getRight());
		}
	}

	//attraversamento INORDER iterativo 
	public void iterativeInorder (IntBSTNode p)
	{
		StackList supp = new StackList ();
		while (p != null)
		{
			if (p.getRight() != null)
				supp.push (p);
			supp.push (p);
			p = p.getLeft();
		}
		p = (IntBSTNode) supp.pop();
		while (!supp.isEmpty() && p.getRight() == null)
		{
			p.visit();
			p = (IntBSTNode) supp.pop();
		}
		p.visit();
		if (!supp.isEmpty())
			p = (IntBSTNode) supp.pop();
		else
			p = null;
	}

	//attraversamento POSTORDER ricorsivo (sottoalbero sx, sottoalbero dx, radice)
	public void postorder (IntBSTNode p)
   {
		if (p != null)
		{
			postorder (p.getLeft());
			postorder (p.getRight());
			p.visit ();
		}
	}

	//attarversamento POSTORDER iterativo
	public void iterativePostorder (IntBSTNode p)
	{
		IntBSTNode q = p;
		StackList supp = new StackList();
		while (p != null)
		{
			for ( ; p.getLeft() != null; p = p.getLeft())
				supp.push (p);
			while (p != null && (p.getRight() == null || p.getRight () == q))
			{
				p.visit();
				q = p;
				if (supp.isEmpty())
					return;
				p = (IntBSTNode) supp.pop();
			}
			supp.push (p);
			p = p.getRight();
		}
	}

	/******************* Metodi per la RICERCA *******************/
	
	//ricerca (iterativa):
	public IntBSTNode iterativeSearch (int val)
	{
		IntBSTNode p = root;
		while (p != null)
		{
			if (p.getKey() == val)
				return p;
			else
			{
				if (val < p.getKey())
					p = p.getLeft();
				else
					p = p.getRight();
			}
		}
		return null;
	}

	public IntBSTNode iterativeSearch (IntBSTNode p, int val)
	{
		while (p != null)
		{
			if (p.getKey() == val)
				return p;
			else
			{
				if (val < p.getKey())
					p = p.getLeft();
				else
					p = p.getRight();
			}
		}
		return null;
	}

	//ricerca (ricorsiva):
	public IntBSTNode search (int val)
	{
		IntBSTNode p = root;
		if (p == null)
			return null;
		if (p.getKey() == val)
			return p;
		else if (val < p.getKey())
			return this.search (p.getLeft(), val);
		else
			return this.search (p.getRight(), val);
	}
	public IntBSTNode search (IntBSTNode p, int val)
	{
		if (p == null)
			return null;
		if (p.getKey() == val)
			return p;
		else if (val < p.getKey())
			return this.search (p.getLeft(), val);
		else
			return this.search (p.getRight(), val);
	}

	/******************* Metodi per l'INSERIMENTO *******************/
	
	//inserimento (iterativo):
	public void insert (int val)
	{
		if (root == null)		//BST vuoto
			root = new IntBSTNode (val);
		else		//BST non vuoto
		{
			IntBSTNode p = root , prev = null;
			while (p != null)
			{
				if (p.getKey() == val)	//l'elemento gi esiste
					throw new NodeFoundException();
				prev = p;
				if (val < p.getKey())
					p = p.getLeft();
				else if (val > p.getKey())
					p = p.getRight();
			}
			// possibile inserire l'elemento:
			if (val < prev.getKey())
				prev.setLeft (new IntBSTNode (val));
			else
				prev.setRight (new IntBSTNode (val));
		}
	}

	//inserimento (ricorsivo):
	public void insert (IntBSTNode p, int val) //overloading!
	{
		if (root == null)
			root = new IntBSTNode (val);
		else if (p.getLeft() == null || p.getRight() == null)
		{
			if (val < p.getKey())
				p.setLeft (new IntBSTNode (val));
			else
				p.setRight (new IntBSTNode (val));
		}
		else
		{
			if (val < p.getKey())
				this.insert (p.getLeft(), val);
			else
				this.insert (p.getRight(), val);
		}
	}

	/******************* Metodo per la CANCELLAZIONE *******************/
	public IntBSTNode delete (int val)
	{
		if (root == null) 	//l'albero  vuoto
			throw new EmptyTreeException();
		IntBSTNode nodo, p = root, prev = null;
		while (p != null && p.getKey() != val) 	//ricerca dell'elemento
		{
			prev = p; 	//tengo il riferimento del precedente
			if (val < p.getKey())
				p = p.getLeft();
			else
				p = p.getRight();
		}
		nodo = p;
		if (p != null && p.getKey() == val) 	// stato trovato l'elemento da cancellare
		{
			if (nodo.getRight() == null) 	//il nodo non ha figlio dx
				nodo = nodo.getLeft();
			else if (nodo.getLeft() == null) 	//il nodo non ha figlio sx
				nodo = nodo.getRight();
			else 	//il nodo ha ambedue i figli
			{
				/*********FONDISOTTOALBERI***********/
				//ricerca del predecessore:
				IntBSTNode tmp = nodo.getLeft();
				while (tmp.getRight() != null)
					tmp = tmp.getRight();
				//fusione:
				tmp.setRight (nodo.getRight());
				nodo = nodo.getLeft();
				/************************************/
				/*********SOSTITUISCICHIAVE**********/
					/*
					//ricerca del predecessore:
					IntBSTNode tmp = nodo.getLeft();
					IntBSTNode previous = nodo;
					while (tmp.getRight() != null)
					{
						previous = tmp;
						tmp = tmp.getRight();
					}
					nodo.setKey (tmp.getKey()); //sostituzione della chiave
					if (previous == nodo)
						previous.setLeft (tmp.getLeft());
					else
						previous.setRight (tmp.getRight());
					*/
					/********************************/
			}
			if (p == root)
				root = nodo;
			else if (p.getLeft() == p)
				prev.setLeft(nodo);
			else
				prev.setRight(nodo);
			return p;
		}
		return null;
	}

	//ESERCIZIO B1: metodo ricorsivo che restituisce l'altezza del BST
	public int getHeight (IntBSTNode p)
	{
		int h = 0;
		if (p == null)
			return 0;
		else
		{
			int sx = 1 + getHeight (p.getLeft());
			int dx = 1 + getHeight (p.getRight());
			h = Math.max (sx, dx);
		}
		if (p == this.getRoot())
			return h-1;
		return h;
	}

	//ESERCIZIO B2: metodo ricorsivo che restituisce il numero di foglie del BST
	public int getNumFoglie (IntBSTNode p)
	{
		int c = 0;
		if (p != null)
		{
			if (p.getLeft() == null && p.getRight() == null)
				return 1;
			else
			{
				c += getNumFoglie (p.getLeft());
				c += getNumFoglie (p.getRight());
			}
		}
		return c;
	}

	//Metodo ricorsivo che restituisce il numero di nodi del BST:
	public int getNumNodi (IntBSTNode p)
	{
		if (p == null)
			return 0;
		else return 1 + getNumNodi (p.getLeft()) + getNumNodi (p.getRight());
	}

	//ESERCIZIO B8: metodo che dato in input un nodo restituisce il nodo fratello
	public IntBSTNode getBrother (IntBSTNode p)
	{
		IntBSTNode aux = root, prev = null;
		while (aux != null)
		{
			if (p == aux)
			{
				if (prev.getRight() == p)
					return prev.getLeft();
				else return prev.getRight();
			}
			prev = aux;
			if (p.getKey() < aux.getKey())
				aux = aux.getLeft();
			else aux = aux.getRight();
		}
		return null;
	}
}

