//Classe IntList per la struttura dati: lista semplicemente concatenata di INTERI

public class IntList
{
	protected IntNode head;	//var d'istanza che tiene il riferimento alla testa della lista

	//Costruttore che inizializza una lista vuota
	public IntList ()
	{
		head = null;	//assegnato il riferimento null
	}

	//metodo imperativo per verificare se la lista  vuota
	public boolean isEmpty ()
	{
		return (head == null);	//restituisce true se la lista  vuota, false altrimenti
	}

	//metodo che restiuisce il nodo di testa della lista
	public IntNode getHead()
	{ 
		return head; 
	}

	//metodo che modifica il riferimento alla testa
	public void setHead (IntNode node)
	{
		head = node;
	}

	//metodo che cancella la lista
	public void clear ()
	{
		head = null; 
	}

	public int getLength ()
	{
		int cont = 1;
		IntNode aux = head;
		for ( ; aux.getNext() != null; cont++, aux = aux.getNext()) ;
		return cont;
	}

	/*		Metodi per l'inserimento	*/
	
	//inserimento in testa:
	public void insertHead (int val)
	{
		/* creo un nuovo nodo
			head diventa il rif al campo next del nuovo nodo
			se la lista  vuota ne viene creata una nuova
		*/
		head = new IntNode (val, head);
	}

	//inserimento in coda: (occorre verificare che la lista non sia vuota)
	public void insertTail (int val)
	{
		//controllo se la lista  vuota
		//in questo caso occorre creare una lista nuova
		if (isEmpty())
		{
			head = new IntNode (val);
		}
		else //se la lista non  vuota occorre scandirla fino alla fine
		{
			//utilizzo un riferimento ausiliario aux per scorrere la lista
			IntNode aux = head;
			for (; aux.getNext() != null; aux = aux.getNext()) ;	//scorro la lista senza 
																					//apportare modifiche
			//aggancio il nuovo nodo all'ultimo
			aux.setNext (new IntNode(val));
		}
	}

	//inserimento ordinato (in un qualunque punto della lista)
	public void insertOrdered (int val)
	{
		//1 caso: la lista  vuota, occorre crearne una nuova
		if (isEmpty())
		{
			head = new IntNode(val);
		}
		//2 caso: l'elemento da inserire  minore del valore del primo nodo
		else if (val < head.getInfo())
		{	//inserimento in testa:
			insertHead (val);
		}
		//3 caso: occorre scandire la lista finch non si trova un el. min. o fino alla fine
		else
		{
			//scansione:
			IntNode aux = head;
			for (; (aux.getNext() != null) && (aux.getNext().getInfo() < val); aux = aux.getNext()) ;
			//trovata la posizione in cui inserire il nodo bidogna collegare il rif.
			aux.setNext (new IntNode(val, aux.getNext()));
		}
	}

	/*		Metodi per la ricerca		*/

	/*
		Primo caso: se la lista  ordinata occorre scandirla fin quando non si trova
		un elemento minore (o maggiore) dell'elemento che si vuole ricercare;
		secondo caso: se la lista non  ordinata occorre scandirla fin quando non si trova
		l'elemento desiderato e, nel caso limite, fino alla sua coda
	*/

	//ricerca ordinata
	public IntNode searchOrd (int key)				 //key  la chiave di ricerca
	{
		IntNode aux = head;
		for (; aux != null && aux.getInfo() < key; aux = aux.getNext()) ;
		if (aux != null && aux.getInfo() == key)
			return aux;
		return null;	//nel caso in cui l'elemento da ricercare non fosse stato trovato
	}

	//ricerca non ordinata
	public IntNode search (int key)
	{
		IntNode aux = head;
		for (; aux != null && aux.getInfo() != key; aux = aux.getNext()) ;
		return aux;
	}

	/*		metodi per il cancellamento	*/
 
	//cancellamento della testa
	//il metodo restituisce il campo info del nodo che si vuole cancellare
	public int deleteHead()
	{
		//se si tenta di cancellare un nodo da una lista vuota viene generate un'eccezione
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			IntNode aux = head;
			//alla testa assegno il riferimento dell'elemento successivo (il secondo)
			head = head.getNext();
			return aux.getInfo();
		}
	}

	//cancellamento della coda
	public int deleteTail ()
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			IntNode aux = head;
			IntNode prev = null;		  //tiene il riferimento del nodo precedente ad aux

			for (; aux.getNext() != null; prev = aux, aux = aux.getNext());
			//1 caso: se prev  null, allora la lista  costituita da un solo elemento
			//quindi eliminare un elemento significa rendere vuota la lista
			if (prev == null)
			{
				head = null;
			}
			//2 caso: la lista  costituita da pi di un elemento
			else
			{
				prev.setNext(null);
			}
			return aux.getInfo();
		}
	}

	//cancellazione di un altro elemento della lista
   public IntNode deleteKey (int key)
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			IntNode aux = head;
			IntNode prev = null;
			for (; aux != null && aux.getInfo() != key; prev = aux, aux = aux.getNext());
			if (aux != null)
			{
				if (prev == null)
				{
					head = head.getNext();
				}
				else
				{
					prev.setNext(aux.getNext());
				}
			}
			return aux;
		}
	}

	//Esercizio A_9:
	public void remove (int pos)
	{
		IntNode aux = head;
		IntNode prev = null;

		try
		{
			for (int i = 0; i < pos; i++)
			{
				prev = aux;
				aux = aux.getNext();
			}
		}
		catch (OutOfListException e)
		{
			return;
		}
		prev.setNext (aux.getNext());
	}

	//Esercizio A_8:
	public void concatena (IntList b)
	{
		IntNode aux = head;
		for ( ; aux.getNext() != null; aux = aux.getNext()) ;
		aux.setNext (b.getHead());
	}
}