//Classe IntListDll per la struttura dati: lista doppiamente linkata di INTERI

public class IntListDll
{
	//per questo tipo di lista occorre avere un rif. alla testa ed uno alla coda
	protected IntNodeDll head, tail;

	//Costruttore (che inizializza una lista vuota)
	public IntListDll ()
	{
		head = tail = null;
	}

	//metodo che verifica se la lista  vuota
	public boolean isEmpty()
	{
		return (head == null);
	}

	public void clear ()
	{
		head = tail = null;
	}

	public int getLength ()
	{
		int cont = 1;
		IntNodeDll aux = head;
		for ( ; aux.getNext() != null; cont++, aux = aux.getNext()) ;
		return cont;
	}

	//metodo che restituisce il riferimento alla testa
	public IntNodeDll getHead()
	{
		return head;
	}

	//metodo che modifica il riferimento alla testa
	public void setHead (IntNodeDll node)
	{
		head = node;
	}

	//metodo che restituisce il riferimento alla coda
	public IntNodeDll getTail()
	{
		return tail;
	}

	//inserimento in testa:
	public void insertHead (int val)
	{
		if (isEmpty())
		{
			head = tail = new IntNodeDll (val);
		}
		else
		{
			head = new IntNodeDll (val, head, null);
			head.getNext().setPrev (head);
		}
	}

	//inserimento in coda (duale del metodo precedente)
	public void insertTail (int val)
	{
		if (isEmpty())
		{
			head = tail = new IntNodeDll (val);
		}
		else
		{
			tail = new IntNodeDll (val, null, tail);
			tail.getPrev().setNext (tail);
		}
	}

	//inserimento ordinato
	public void insertOrdered (int val)
	{
		//1 caso: la lista  vuota
		if (isEmpty())
		{
			//creo una nuova lista
			head = tail = new IntNodeDll (val);
		}
		//2 caso: l'elemento va inserito in testa alla lista
		else if (val <= head.getInfo())
		{
			insertHead (val);
		}
		//3 caso: l'elemento va inserito in mezzo o, caso limite, alla coda della lista
		else 
		{
			//scansione della lista:
			IntNodeDll aux = head;
			for ( ; (aux != null) && (aux.getInfo() > val); aux = aux.getNext()) ;
			if (aux == null) 	//allora inserisci in coda
			{
				insertTail(val);
			}
			else
			{
				aux.setPrev (new IntNodeDll (val, aux, aux.getPrev()));
				aux.getPrev().getPrev().setNext(aux.getPrev());
			}
		}
	}

	//i metodi di ricerca sono identici a quelli delle liste linkate semplici:
	//ricerca ordinata
	public IntNodeDll searchOrd (int key)				 //key  la chiave di ricerca
	{
		IntNodeDll aux = head;
		for (; aux != null && aux.getInfo() < key; aux = aux.getNext()) ;
		if (aux != null && aux.getInfo() == key)
			return aux;
		return null;	//nel caso in cui l'elemento da ricercare non fosse stato trovato
	}

	//ricerca non ordinata
	public IntNodeDll search (int key)
	{
		IntNodeDll aux = head;
		for (; aux != null && aux.getInfo() != key; aux = aux.getNext()) ;
		return aux;
	}

	/*		Metodi per il cancellamento	*/

	//cancellamento del primo elemento
	//in ogni caso occorre SEMPRE aggiornare sia il campo next che prev
	public int deleteHead ()
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else 
		{
			IntNodeDll aux = head;
			if (head == tail)	//se la lista  costituita da un solo elemento
			{
				head = tail = null;
			}
			else //la lista ha pi di un elemento
			{
				head = head.getNext();
				head.setPrev (null);
			}
			return aux.getInfo();
		}
	}

	//cancellamento dell'ultimo elemento (duale del metodo precdedente)
	public int deleteTail ()
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			IntNodeDll aux = tail;
			if (head == tail)
			{
				head = tail = null;
			}
			else 
			{
				tail = tail.getPrev();
				tail.setNext (null);
			}
			return aux.getInfo();
		}
	}

	//cancellamento di una chiave specifica
	public IntNodeDll deleteKey (int key) 
	{
		IntNodeDll aux = head;
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		for ( ; aux != null && aux.getInfo() != key; aux = aux.getNext()) ;
		if (aux != null)
		{
			if (aux.getPrev() == null)
			{
				if (head == tail)
				{
					head = tail = null;
				}
				else 
				{
					head = head.getNext();
					head.setPrev(null);
				}
			}
			else
			{
				if (aux == tail)
				{
					tail = tail.getPrev();
					tail.setNext(null);
				}
				else 
				{
					aux.getPrev().setNext (aux.getNext());
					aux.getNext().setPrev (aux.getPrev());
				}
			}
		}
		return aux;
	}
}