//Classe List per la struttura dati: List generica semplicemente concatenata

public class List
{
	protected Node head;

	//Costruttore:
	public List ()
	{
		head = null;	//inizializzo una List generica vuota
	}

	//metodo che restituisce il riferimento alla testa della lista
	public Node getHead()
	{
		return head;
	}

	//metodo che modifica il riferimento alla testa
	public void setHead (Node node)
	{
		head = node;
	}
	
	public int getLength ()
	{
		int cont = 0;
		Node aux = head;
		for ( ; aux.getNext() != null; cont++, aux = aux.getNext()) ;
		return cont;
	}

	//metodo che cancella l'intera lista
	public void clear ()
	{
		head = null; }

	//metodo che controlla se la List  vuota
	public boolean isEmpty ()
	{
		return (head == null);
	}

	//metodo che stampa a schermo tutti gli elementi della List
	public String printAll ()
	{
		String output = "";
		if (isEmpty())
		{
			output = "List empty\n";
		}
		else
		{
			Node aux = head;
			for (; aux.getNext() != null; aux = aux.getNext())
			{
				output += (aux.getInfo()).toString() + "\n";
			}
			output += (aux.getInfo()).toString() + "\n";
		}
		return output;
	}

	/*		Metodi per l'inserimento	*/

	//inserimento in testa
	public void insertHead (Object o)
	{
		head = new Node (o, head);
	}

	//inserimento in coda
	public void insertTail (Object o)
	{
		//controllo se la List  vuota
		if (isEmpty())
		{
			//creo una nuova List
			head = new Node (o);
		}
		else
		{
			Node aux = head;
			for (; aux.getNext() != null; aux = aux.getNext());
			aux.setNext (new Node (o));
		}
	}

	//inserimento ordinato (in qualunque punto della List)
	public void insertOrdered (Object o)
	{
		//controllo se la List  vuota
		if (isEmpty())
		{
			head = new Node (o, head);
		}
		//altrimenti controllo se l'elemento da inserire  minore dell'elemento di testa
		else if (((Comparable)head.getInfo()).compareTo(o) > 0)
		{
			//inserimento in testa
			insertHead (o);
		}
		//come ultima alternativa devo scorrere la List fino alla posizione corretta
		else
		{
			//scansione:
			Node aux = head;
			for (; aux.getNext() != null && ((Comparable)aux.getInfo()).compareTo(o) < 0; 
					 aux = aux.getNext());
			//posso "agganciare" l'elemento
			aux.setNext(new Node(o, aux.getNext()));
		}
	}

	/*		Metodi per la ricerca	*/
	
	//ricerca ordinata
	public Node searchOrd (Object key)
	{
		//scansione della List:
		Node aux = head;
		for ( ; aux.getNext() != null && ((Comparable)aux.getInfo()).compareTo(key) < 0;
				  aux = aux.getNext()) ;
		if (aux != null && (aux.getInfo().equals(key)))
			return aux;
		return null;
	}

	//ricerca non ordinata
	public Node search (Object key)
	{
		Node aux = head;
		for ( ; aux != null && !(aux.getInfo()).equals(key); aux = aux.getNext()) ;
		return aux;
	}

	/*		Metodi per il cancellamento	*/

	//cancellamento della testa
	public Object deleteHead ()
	{
		//se la List  vuota genero un'eccezione di tipo EmptyListException
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			Node aux = head;
			head = head.getNext();
			return aux.getInfo();
		}
	}

	//cancellamento della coda
	public Object deleteTail ()
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			Node aux = head;
			Node prev = null;
			for ( ; aux.getNext() != null; prev = aux, aux = aux.getNext()) ;
			if (prev == null) //allora la List  costituita da un solo elemento
			{
				prev = null;
			}
			else
			{
				prev.setNext(null);
			}
			return aux.getInfo();
		}
	}

	//cancellamento di un ben preciso elemento della List
	public Node deleteKey (Object key)
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			Node aux = head;
			Node prev = null;
			for ( ; aux != null && !(aux.getInfo().equals(key)); prev = aux, aux = aux.getNext()) ;
			if (aux != null)
			{
				if (prev == null)
				{
					head = head.getNext();
				}
				else
				{
					prev.setNext(aux.getNext());
				}
			}
			return aux;
		}
	}

	//cancellamento totale
	public void deleteAll()
	{
		head = null;
	}
}