//Classe ListaDbl per la struttura dati: lista doppiamente linkata generica

public class ListDll
{
	protected NodeDll head, tail;

	public ListDll ()
	{
		head = tail = null;
	}

	public boolean isEmpty ()
	{
		return (head == null);
	}

	public void clear ()
	{
		head = tail = null;
	}

	//metodo che restituisce il riferimento alla testa
	public NodeDll getHead()
	{
		return head;
	}

	//metodo che modifica il riferimento alla testa
	public void setHead (NodeDll node)
	{
		head = node;
	}

	//metodo che restituisce il riferimento alla coda
	public NodeDll getTail()
	{
		return tail;
	}

	public int getLength ()
	{
		int cont = 1;
		NodeDll aux = head;
		for ( ; aux.getNext() != null; cont++, aux = aux.getNext()) ;
		return cont;
	}

	//inserimento in testa
	public void insertHead (Object obj)
	{
		if (isEmpty())
		{
			head = tail = new NodeDll (obj);
		}
		else
		{
			head = new NodeDll (obj, head, null);
			head.getNext().setPrev (head);
		}
	}

	//inserimento in coda
	public void insertTail (Object obj)
	{
		if (isEmpty())
		{
			head = tail = new NodeDll (obj);
		}
		else 
		{
			tail = new NodeDll (obj, null, tail);
			tail.getPrev().setNext (tail);
		}
	}

	//inserimento ordinato
	public void insertOrdered (Object obj)
	{
		//1 caso: la lista  vuota
		if (isEmpty())
		{
			head = tail = new NodeDll (obj);
		}
		//2 caso: l'elemento va inserito in testa alla lista
		else if (((Comparable)head.getInfo()).compareTo(obj) > 0)
		{
			insertHead(obj);
		}
		//3 caso: l'elemento va inserito in mezzo o, caso limite nella coda della lista
		else
		{
			//scansione della lista
			NodeDll aux = head;
			for ( ; (aux != null) && (((Comparable)aux.getInfo()).compareTo(obj)) > 0; aux = aux.getNext()) ;
			if (aux == null)
			{
				insertTail (obj);
			}
			else 
			{
				aux.setPrev (new NodeDll (obj, aux, aux.getPrev()));
				aux.getPrev().getPrev().setNext(aux.getPrev());
			}
		}
	}

	//ricerca ordinata
	public NodeDll searchOrd (Object key)
	{
		NodeDll aux = head;
		for ( ; (aux != null) && (((Comparable)aux.getInfo()).compareTo(key) < 0); aux = aux.getNext()) ;
		if (aux != null && aux.getInfo().equals(key))
		{
			return aux;
		}
		return null;
	}

	//ricerca non ordinata
	public NodeDll search (Object key)
	{
		NodeDll aux = head;
		for ( ; aux != null && !(aux.getInfo().equals(key)); aux = aux.getNext()) ;
		return aux;
	}

	//cancellamento del primo elemento
	public Object deleteHead ()
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			NodeDll aux = head;
			if (head == tail)
			{
				head = tail = null;
			}
			else 
			{
				head = head.getNext();
				head.setPrev (null);
			}
			return aux.getInfo();
		}
	}

	//cancellamento dell'ultimo elemento
	public Object deleteTail ()
	{
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		else
		{
			NodeDll aux = head;
			if (head == tail)
			{
				head = tail = null;
			}
			else 
			{
				tail = tail.getPrev();
				tail.setNext (null);
			}
			return aux.getInfo();
		}
	}

	//cancellamento di una chiave specifica
	public NodeDll deleteKey (Object key) 
	{
		NodeDll aux = head;
		if (isEmpty())
		{
			throw new EmptyListException();
		}
		for ( ; aux != null && aux.getInfo() != key; aux = aux.getNext()) ;
		if (aux != null)
		{
			if (aux.getPrev() == null)
			{
				if (head == tail)
				{
					head = tail = null;
				}
				else 
				{
					head = head.getNext();
					head.setPrev(null);
				}
			}
			else
			{
				if (aux == tail)
				{
					tail = tail.getPrev();
					tail.setNext(null);
				}
				else 
				{
					aux.getPrev().setNext (aux.getNext());
					aux.getNext().setPrev (aux.getPrev());
				}
			}
		}
		return aux;
	}
}